<?php

namespace Passimpay\PassimpayPayment\Controller\Checkout;

use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;

class Notify extends \Magento\Framework\App\Action\Action implements CsrfAwareActionInterface
{
    /**
     * @var \Passimpay\PassimpayPayment\Model\Config
     */
    protected $_config;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var PaymentHelper
     */
    protected $_paymentHelper;

    /**
     * @var TransactionBuilder
     */
    protected $_transactionBuilder;

    /**
     * @var Order
     */
    protected $_order;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $_orderRepository;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $_invoiceService;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\InvoiceSender
     */
    protected $_invoiceSender;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\OrderSender
     */
    protected $_orderSender;


    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Sales\Model\Order\Payment\Transaction\Builder $trans,
        \Magento\Sales\Api\Data\OrderInterface $order,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
   )
    {
        $this->_checkoutSession = $checkoutSession;
        $this->_logger = $logger;
        $this->_paymentHelper = $paymentHelper;
        $this->_config = $scopeConfig;
        $this->_transactionBuilder = $trans;
        $this->_order = $order;
        $this->_invoiceService = $invoiceService;
        $this->_invoiceSender = $invoiceSender;
        $this->_orderRepository = $orderRepository;
        $this->_orderSender = $orderSender;

        parent::__construct($context);
    }

    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    public function execute()
    {

//        throw new \Exception(json_encode([
//            'page' => __CLASS__,
//            'input' => file_get_contents("php://input"),
//            'get' => $_GET,
//            'post' => $_POST,
//        ]));

        $params = $_POST;

        $order = $this->_orderRepository->get($params['order_id']);

        $method = $order->getPayment()->getMethod();

        $methodInstance = $this->_paymentHelper->getMethodInstance($method);

        $data = [
            'platform_id' => (int) $_POST['platform_id'], // Platform ID
            'payment_id' => (int) $_POST['payment_id'], // currency ID
            'order_id' => (int) $_POST['order_id'], // Payment ID of your platform
            'amount' => $_POST['amount'], // transaction amount
            'txhash' => $_POST['txhash'], // Hash or transaction ID. You can find the transaction ID in the PassimPay transaction history in your account.
            'address_from' => $_POST['address_from'], // sender address
            'address_to' => $_POST['address_to'], // recipient address
            'fee' => $_POST['fee'], // network fee
        ];

        if (!$methodInstance->checkHash($data, $params['hash'])) {
            die('NOT OK');
        }

        if ($order->getState() !== $order::STATE_PENDING_PAYMENT) {
            die('Order status is not fit');
        }

        if (!$methodInstance->orderStatusIsCompleted($order->getId())) {
            die('Payment process has wrong status');
        }

        $paymentId = $params["payment_id"];
        $amount = $params["amount"];

        /** @var \Magento\Sales\Model\Order\Status $status */
        $order->setState($order::STATE_COMPLETE)
            ->setStatus($order->getConfig()->getStateDefaultStatus($order::STATE_COMPLETE))
            ->save();

        // Send order notification
        try {
            $this->_orderSender->send($order);
        } catch (\Exception $e) {
            $this->_objectManager->get('Psr\Log\LoggerInterface')->critical($e);
        }

        //get payment object from order object
        $payment = $order->getPayment();
        $payment->setLastTransId($paymentId);
        $payment->setTransactionId($paymentId);

        $message = __('Captured amount is %1 USD.', $amount);

        //get the object of builder class
        $trans = $this->_transactionBuilder;
        $transaction = $trans->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($paymentId)
            ->setFailSafe(true)
            //build method creates the transaction and returns the object
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);

        $payment->addTransactionCommentsToOrder(
            $transaction,
            $message
        );
        $payment->setParentTransactionId(null);
        $payment->save();
        $order->save();

        $transId = $transaction->save()->getTransactionId();

        $message = __('Payment has been captured.');

        // Create Invoice for Sale Transaction
        $invoice = $this->makeInvoice($order, [], false, $message);
        $invoice->setTransactionId($transId);
        $invoice->save();

        die('OK');


//        throw new \Exception(json_encode([
//            'orderRepo' => $methodInstance->checkHash($data, $params['hash']),
//            'order' => $order->getQuoteId(),
//            'method' => $method,
//            'instance' => $methodInstance,
//            'params' => $params,
//        ]));
    }

    /**
     * Create Invoice
     * @param \Magento\Sales\Model\Order $order
     * @param array $qtys
     * @param bool $online
     * @param string $comment
     * @return \Magento\Sales\Model\Order\Invoice
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function makeInvoice(\Magento\Sales\Model\Order $order, array $qtys = [], $online = false, $comment = '')
    {
        /** @var \Magento\Framework\ObjectManagerInterface $om */
        $om = \Magento\Framework\App\ObjectManager::getInstance();

        /** @var \Magento\Sales\Model\Order\Invoice $invoice */
        $invoice = $this->_invoiceService->prepareInvoice($order, $qtys);
        $invoice->setRequestedCaptureCase($online ? \Magento\Sales\Model\Order\Invoice::CAPTURE_ONLINE : \Magento\Sales\Model\Order\Invoice::CAPTURE_OFFLINE);

        // Add Comment
        if (!empty($comment)) {
            $invoice->addComment(
                $comment,
                true,
                true
            );

            $invoice->setCustomerNote($comment);
            //$invoice->setCustomerNoteNotify(true);
        }

        $invoice->register();
        $invoice->getOrder()->setIsInProcess(true);

        // send invoice emails
        try {
            $this->_invoiceSender->send($invoice);
        } catch (\Exception $e) {
            $om->get('Psr\Log\LoggerInterface')->critical($e);
        }

        $invoice->setIsPaid(true);

        // Assign Last Transaction Id with Invoice
        $transactionId = $invoice->getOrder()->getPayment()->getLastTransId();
        if ($transactionId) {
            $invoice->setTransactionId($transactionId);
            $invoice->save();
        }

        return $invoice;
    }

    /**
     * Return checkout session object
     *
     * @return \Magento\Checkout\Model\Session
     */
    protected function _getCheckoutSession()
    {
        return $this->_checkoutSession;
    }
}
