<?php

namespace Passimpay\PassimpayPayment\Model;

class PassimpayPaymentMethod extends \Magento\Payment\Model\Method\AbstractMethod
{

    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = 'passimpaypaymentmethod';
    protected $_isGateway = true;

    protected $_countryFactory;

    protected $_supportedCurrencyCodes = [
        'USD',
    ];

    protected $_storeManager;

    protected $logger;

    /**
     * @var \Passimpay\PassimpayPayment\Helper\PassimpayMerchantAPI
     */
    private $helper;

    /**
     * Payment data
     *
     * @var \Magento\Payment\Helper\Data
     */
    protected $_paymentData;

    /**
     * Core store config
     *
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Passimpay\PassimpayPayment\Helper\PassimpayMerchantAPI $helper,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            null,
            null,
            $data
        );

        $this->helper = $helper;
        $this->helper->setApiUrl($this->getConfigData('gateway'));
        $this->helper->setPlatformId(intval($this->getConfigData('platform_id')));
        $this->helper->setSecretKey($this->getConfigData('secret_key'));

        $this->_storeManager = $storeManager;
    }

    public function startTransaction(\Magento\Sales\Model\Order $order, \Magento\Framework\UrlInterface $url)
    {
        $arrBillingAddress = $order->getBillingAddress()->toArray();
        $email = $arrBillingAddress['email'];

        $arrFields = array(
            'order_id'  => intval($order->getRealOrderId()),
            'amount'   => number_format($order->getGrandTotal(), 2, '.', ''),
        );

        $request = $this->helper->buildQuery('createorder', $arrFields);


        if ($request->paymentUrl) {

            $order->addStatusHistoryComment(
                __('The customer was redirected to Passimpay.'),
                \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT
            );
            $order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT)
                ->setStatus($order->getConfig()->getStateDefaultStatus(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT));
            $order->save();

            return $request->paymentUrl;
        } else {
            throw new \Magento\Framework\Exception\LocalizedException(__('Cannot get payment url from Passimpay.'));
        }

    }

    /**
     * Проверяем, оплачен ли заказ
     *
     * @param $orderId
     * @return bool
     */
    public function orderStatusIsCompleted($orderId)
    {
        return $this->helper->getOrderStatus($orderId) === $this->helper::PAYMENT_STATUS_COMPLETED;
    }

    /**
     * Validate params from request
     *
     * @param $params
     * @param $hash
     * @return bool
     */
    public function checkHash($params, $hash)
    {
        $apikey = $this->helper::getSecretKey();

        $payload = http_build_query($params);

        if (!isset($hash) || hash_hmac('sha256', $payload, $apikey) != $hash)
        {
            return false;
        }

        return true;
    }

    /**
     * Determine method availability based on quote amount and config data
     *
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if (!$this->getConfigData('platform_id') || !$this->getConfigData('secret_key')) {
            return false;
        }

        return parent::isAvailable($quote);
    }

    public function getMerchantData($data = "")
    {
        return $this->helper->__get($data);
    }


    /**
     * Availability for currency
     *
     * @param string $currencyCode
     * @return bool
     */
    public function canUseForCurrency($currencyCode)
    {
        $currencyCode = $this->_storeManager->getStore()->getCurrentCurrency()->getCode();
        if (!in_array($currencyCode, $this->_supportedCurrencyCodes)) {
            return false;
        }
        return true;
    }
}
